/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.dom.dfl;

import de.uniko.bks.dom.dfl.rule.Comparator;
import de.uniko.bks.dom.dfl.rule.RuleVariable;

/** Class specifying how a date reasonable in SPINdle looks like */
public class Date {
  /** Year of a date (four digits: YYYY) */
  private String year;
  /** Month of a date (two digits: MM) */
  private String month;
  /** Day of a date (two digits: DD) */
  private String day;
  /** Hour of a date (two digits: hh) */
  private String hour;
  /** Minute of a date (two digits: ii) */
  private String minute;
  /** Second of the date (two digits: ss) */
  private String second;
  /** Comparator used for comparisons of dates */
  private String comparator;

  /** Constructor for an empty date */
  public Date() {
    this.year = "";
    this.month = "";
    this.day = "";
    this.hour = "";
    this.minute = "";
    this.second = "";
    this.comparator = Comparator.EQUALS.getValue();
  }

  /**
   * Constructor for a date without a time
   *
   * @param year Year of a date (four digits: YYYY)
   * @param month Month of a date (two digits: MM)
   * @param day Day of a date (two digits: DD)
   */
  public Date(String year, String month, String day) {
    this.year = year;
    this.month = month;
    this.day = day;
    this.hour = "00";
    this.minute = "00";
    this.second = "00";
    this.comparator = Comparator.EQUALS.getValue();
  }

  /**
   * Constructor for a date with a time
   *
   * @param year Year of a date (four digits: YYYY)
   * @param month Month of a date (two digits: MM)
   * @param day Day of a date (two digits: DD)
   * @param hour Hour of a date (two digits: hh)
   * @param minute Minute of a date (two digits: ii)
   * @param second Second of the date (two digits: ss)
   */
  public Date(String year, String month, String day, String hour, String minute, String second) {
    this.year = year;
    this.month = month;
    this.day = day;
    this.hour = hour;
    this.minute = minute;
    this.second = second;
    this.comparator = Comparator.EQUALS.getValue();
  }

  /**
   * Getter for the date comparator
   *
   * @return The comparator used for date comparisons
   */
  public String getComparator() {
    return comparator;
  }

  /**
   * Setter for the date comparator
   *
   * @param comparator The comparator used for date comparisons
   */
  public void setComparator(String comparator) {
    this.comparator = comparator;
  }

  @Override
  public String toString() {
    return RuleVariable.VAR_SIGN.getValue()
        + "date("
        + year
        + ','
        + month
        + ','
        + day
        + ','
        + hour
        + ','
        + minute
        + ','
        + second
        + ')';
  }
}
