/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.dom.dfl;

/**
 * Class representing a fact of any type
 *
 * @param <T> Java type of the fact
 */
public class Fact<T> {
  /** Name of the fact */
  private String name;
  /** Value of the fact */
  private T value;
  /** Java class the value of the fact belongs to */
  private Class<T> type;

  /** Constructor for an empty fact */
  public Fact() {}

  /**
   * Constructor for a completely set fact
   *
   * @param name Name of the fact
   * @param value Value of the fact
   * @param type Java class the value of the fact belongs to
   */
  public Fact(String name, T value, Class<T> type) {
    this.name = name;
    this.value = value;
    this.type = type;
  }

  /**
   * Getter for the fact name
   *
   * @return The name of the fact
   */
  public String getName() {
    return name;
  }

  /**
   * Setter for the fact name
   *
   * @param name The name of the fact
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * Getter for the value of the fact
   *
   * @return The value of the fact
   */
  public T getValue() {
    return value;
  }

  /**
   * Getter for the value of the fact
   *
   * @param value The value of the fact
   */
  public void setValue(T value) {
    this.value = value;
  }

  /**
   * Getter for the type of the fact
   *
   * @return The class name of the type of the fact
   */
  public Class<T> getType() {
    return type;
  }

  /**
   * Setter for the type of the fact
   *
   * @param type The class name of the type of the fact
   */
  public void setType(Class<T> type) {
    this.type = type;
  }
}
