/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.dom.dfl;

/**
 * Class which represents a wrapper for literals containing a range
 *
 * @param <T> Type of the limits of the range
 */
public class LiteralRange<T> {
  /** Whether or not the literal contains a range */
  private boolean range;
  /** Comparator for the lower limit of the range */
  private String lowerLimitComparator;
  /** Comparator for the upper limit of the range */
  private String upperLimitComparator;
  /** Lower limit of the range */
  private T lowerLimit;
  /** Upper limit of the range */
  private T upperLimit;

  /**
   * Construtor for an empty range
   *
   * @param range Whether or not the literal contains a range
   */
  public LiteralRange(boolean range) {
    this.range = range;
  }

  /**
   * Getter for the range
   *
   * @return Whether or not the literal contains a range
   */
  public boolean isRange() {
    return range;
  }

  /**
   * Getter for the lower limit comparator
   *
   * @return The lower limit comparator
   */
  public String getLowerLimitComparator() {
    return lowerLimitComparator;
  }

  /**
   * Setter for the lower limit comparator
   *
   * @param lowerLimitComparator The lower limit comparator
   */
  public void setLowerLimitComparator(String lowerLimitComparator) {
    this.lowerLimitComparator = lowerLimitComparator;
  }

  /**
   * Getter for the upper limit comparator
   *
   * @return The upper limit comparator
   */
  public String getUpperLimitComparator() {
    return upperLimitComparator;
  }

  /**
   * Setter for the upper limit comparator
   *
   * @param upperLimitComparator The upper limit comparator
   */
  public void setUpperLimitComparator(String upperLimitComparator) {
    this.upperLimitComparator = upperLimitComparator;
  }

  /**
   * Getter for the lower limit
   *
   * @return The lower limit
   */
  public T getLowerLimit() {
    return lowerLimit;
  }

  /**
   * Setter for the lower limit
   *
   * @param lowerLimit The lower limit
   */
  public void setLowerLimit(T lowerLimit) {
    this.lowerLimit = lowerLimit;
  }

  /**
   * Getter for the upper limit
   *
   * @return The upper limit
   */
  public T getUpperLimit() {
    return upperLimit;
  }

  /**
   * Setter for the upper limit
   *
   * @param upperLimit The upper limit
   */
  public void setUpperLimit(T upperLimit) {
    this.upperLimit = upperLimit;
  }
}
