/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.dom.dfl.rule;

/** Enum containing all mathematical comparators for comparison expressions */
public enum Comparator {
  /** Comparator for smaller comparisons */
  SMALLER("<"),
  /** Comparator for smaller or equals comparisons */
  SMALLER_OR_EQUALS("<="),
  /** Comparator for equals comparisons */
  EQUALS("=="),
  /** Comparator for greater or equals comparisons */
  GREATER_OR_EQUALS(">="),
  /** Comparator for greater comparisons */
  GREATER(">");
  /** Actual value of the enum */
  private String value;

  /**
   * Constructor to set the value of each enum entry
   *
   * @param value String value of the enum
   */
  Comparator(String value) {
    this.value = value;
  }

  /**
   * Getter for the (string) value of one enum entry
   *
   * @return The value of the enum entry
   */
  public String getValue() {
    return value;
  }

  /**
   * Function which gets the opposite of a given comparator
   *
   * @param comparator Comparator or string containing the comparator
   * @return The comparator which describes the opposite set
   */
  public static String getOppositeValue(String comparator) {
    if (comparator.contains(SMALLER_OR_EQUALS.value)) {
      return GREATER.value;
    }
    if (comparator.contains(GREATER_OR_EQUALS.value)) {
      return SMALLER.value;
    }
    if (comparator.contains(SMALLER.value)) {
      return GREATER_OR_EQUALS.value;
    }
    if (comparator.contains(GREATER.value)) {
      return SMALLER_OR_EQUALS.value;
    }

    return EQUALS.value;
  }
}
