/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.dom.dmn;

import de.uniko.bks.dom.dmn.header.OutputHeader;
import java.util.ArrayList;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlType;

/** Class specifying a dmn DecisionTable */
@XmlType(propOrder = {"input", "outputHeader", "dmnRule"})
public class DecisionTable {
  /** XML id of the decision table */
  private String id;
  /** HitPolicy of the table */
  private String hitPolicy;
  /** XML input elements of the table */
  private ArrayList<Input> input;
  /** OutputHeader (meta-information about the output) of the table */
  private OutputHeader outputHeader;
  /** Rules contained in the table */
  private ArrayList<DmnRule> dmnRule;

  /** Constructor to initialize an empty DecisionTable */
  public DecisionTable() {}

  /**
   * Getter for the XML id of the decision table
   *
   * @return The XML id of the decision table
   */
  @XmlAttribute
  public String getId() {
    return id;
  }

  /**
   * Setter for the XML id of the decision table
   *
   * @param id The XML id of the decision table
   */
  public void setId(String id) {
    this.id = id;
  }

  /**
   * Getter for the HitPolicy of a table
   *
   * @return The HitPolicy of the table
   */
  @XmlAttribute
  public String getHitPolicy() {
    return hitPolicy;
  }

  /**
   * Setter for the HitPolicy of a table
   *
   * @param hitPolicy The HitPolicy of the table
   */
  public void setHitPolicy(String hitPolicy) {
    this.hitPolicy = hitPolicy;
  }

  /**
   * Getter for the XML input elements of the table
   *
   * @return The XML input elements of the table
   */
  @XmlElement(name = "input")
  public ArrayList<Input> getInput() {
    return input;
  }

  /**
   * Getter for the OutputHeader (meta-information about the output) of the table
   *
   * @return The OutputHeader (meta-information about the output) of the table
   */
  @XmlElement(name = "output")
  public OutputHeader getOutputHeader() {
    return outputHeader;
  }
  /**
   * Setter for the XML input elements of the table
   *
   * @param input The XML input elements of the table
   */
  public void setInput(ArrayList<Input> input) {
    this.input = input;
  }

  /**
   * Getter for the rules contained in the table
   *
   * @return The rules contained in the table
   */
  @XmlElement(name = "rule")
  public ArrayList<DmnRule> getDmnRule() {
    return dmnRule;
  }

  /**
   * Setter for the rules contained in the table
   *
   * @param dmnRule The rules contained in the table
   */
  public void setDmnRule(ArrayList<DmnRule> dmnRule) {
    this.dmnRule = dmnRule;
  }

  /**
   * Setter for the OutputHeader (meta-information about the output) of the table
   *
   * @param outputHeader The OutputHeader (meta-information about the output) of the table
   */
  public void setOutputHeader(OutputHeader outputHeader) {
    this.outputHeader = outputHeader;
  }
}
