/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.dom.dmn;

/** Enum containing all supported hit policies for DMN decision tables */
public enum HitPolicy {
  /** Entry for the ANY hit policy */
  ANY("ANY"),
  /** Entry for the COLLECT hit policy */
  COLLECT("COLLECT"),
  /** Entry for the FIRST hit policy */
  FIRST("FIRST"),
  /** Entry for the OUTPUT ORDER hit policy */
  OUTPUT_ORDER("OUTPUT ORDER"),
  /** Entry for the PRIORITY hit policy */
  PRIORITY("PRIORITY"),
  /** Entry for the RULE ORDER hit policy */
  RULE_ORDER("RULE ORDER"),
  /** Entry for the UNIQUE hit policy */
  UNIQUE("UNIQUE");
  /** Actual value of the enum */
  private String value;

  /**
   * Constructor to set the value of each enum entry
   *
   * @param value String value of the enum
   */
  HitPolicy(String value) {
    this.value = value;
  }

  /**
   * Getter for the (string) value of one enum entry
   *
   * @return The value of the enum entry
   */
  public String getValue() {
    return value;
  }

  /**
   * Getter for the (string) value of the default enum entry
   *
   * @return The value of the default enum entry
   */
  public static String getDefault() {
    return UNIQUE.value;
  }
}
