/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.dom.dmn;

import de.uniko.bks.dom.dmn.header.InputHeader;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;

/** Class specifying a DMN Input */
public class Input {
  /** XML id of the Input */
  private String id;
  /** XML label of the Input */
  private String label;
  /** InputHeader of the Input element */
  private InputHeader inputHeader;

  /** Constructor to initialize an empty Input */
  public Input() {}

  /**
   * Constructor to initialize a completely set Input
   *
   * @param id The XML id of the Input
   * @param label The XML label of the Input
   * @param inputHeader The InputHeader of the Input element
   */
  public Input(String id, String label, InputHeader inputHeader) {
    this.id = id;
    this.label = label;
    this.inputHeader = inputHeader;
  }

  @XmlAttribute
  public String getId() {
    return id;
  }

  public void setId(String id) {
    this.id = id;
  }

  /**
   * Getter for the XML label of the Input
   *
   * @return The XML label of the Input
   */
  @XmlAttribute
  public String getLabel() {
    return label;
  }

  /**
   * Setter for the XML label of the Input
   *
   * @param label The XML label of the Input
   */
  public void setLabel(String label) {
    this.label = label;
  }

  /**
   * Getter for the InputHeader of the Input element
   *
   * @return The InputHeader of the Input element
   */
  @XmlElement(name = "inputExpression")
  public InputHeader getInputHeader() {
    return inputHeader;
  }

  /**
   * Setter for the InputHeader of the Input element
   *
   * @param inputHeader The InputHeader of the Input element
   */
  public void setInputHeader(InputHeader inputHeader) {
    this.inputHeader = inputHeader;
  }
}
