/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.dom.dmn.entry;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;

/** Class specifying a dmn InputEntry */
public class InputEntry {
  /** XML id of the InputEntry */
  private String id;
  /** Text/value of the InputEntry */
  private String text;

  /** Constructor for an empty InputEntry */
  public InputEntry() {}

  /**
   * Constructor for a completely initialized InputEntry
   *
   * @param id XML id of the InputEntry
   * @param text Text/value of the InputEntry
   */
  public InputEntry(String id, String text) {
    this.id = id;
    this.text = text;
  }

  /**
   * Getter for the InputEntry id
   *
   * @return The id of the InputEntry
   */
  @XmlAttribute
  public String getId() {
    return id;
  }

  /**
   * Setter for the InputEntry id
   *
   * @param id The id of the InputEntry
   */
  public void setId(String id) {
    this.id = id;
  }

  /**
   * Getter for the InputEntry text/value
   *
   * @return The text/value of the InputEntry
   */
  @XmlElement
  public String getText() {
    return text;
  }

  /**
   * Setter for the InputEntry text/value
   *
   * @param text The text/value of the InputEntry
   */
  public void setText(String text) {
    this.text = text;
  }
}
