/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.dom.dmn.header;

import de.uniko.bks.util.StringSanitizer;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;

/** Class specifying the OutputValues of a dmn table header */
public class OutputValues {
  /** The XML id of the OutputValues */
  private String id;
  /** The text/value of the OutputValues as comma separated list */
  private String text;

  /** Constructor for empty OutputValues */
  public OutputValues() {}

  /**
   * Constructor for completely initialized OutputValues
   *
   * @param id The XML id of the OutputValues
   * @param text The text/value of the OutputValues as comma separated list
   */
  public OutputValues(String id, String text) {
    this.id = id;
    this.text = text;
  }

  /**
   * Getter for the XML id of the OutputValues
   *
   * @return The XML id of the OutputValues
   */
  @XmlAttribute
  public String getId() {
    return id;
  }

  /**
   * Setter for the XML id of the OutputValues
   *
   * @param id The XML id of the OutputValues
   */
  public void setId(String id) {
    this.id = id;
  }

  /**
   * Getter for the text/value of the OutputValues
   *
   * @return The text/value of the OutputValues
   */
  @XmlElement
  public String getText() {
    return text;
  }

  /**
   * Setter for the text/value of the OutputValues
   *
   * @param text The text/value of the OutputValues
   */
  public void setText(String text) {
    this.text = StringSanitizer.sanitize(text, false, true);
  }
}
