/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.generator.entry;

import de.uniko.bks.dom.dmn.entry.InputEntry;
import java.util.Random;

/** Class which generates a single input entry for the DMN */
public class InputEntryGenerator {

  private static String generateInputRange() {
    String out = "";
    if (new Random().nextInt(1000) < 10) {
      out += new Random().nextBoolean() ? "<" : "<=";
      out += new Random().nextInt(1500) - 1000;
    } else if (new Random().nextInt(1000) < 10) {
      out += new Random().nextBoolean() ? ">" : "=";
      out += new Random().nextInt(1500) + 50;
    } else {
      // Value between 0 and 15
      int lowerLimit = new Random().nextInt(2000) - 1500;
      // Value between 15 and 45
      int upperLimit = lowerLimit + new Random().nextInt(1000) + 20;

      out += new Random().nextBoolean() ? "[" : "]";
      out += lowerLimit + ".." + upperLimit;
      out += new Random().nextBoolean() ? "[" : "]";
    }
    return out;
  }

  /**
   * Function which generates an integer input entry range
   *
   * @param identifier Identifier suffix for all ids in the DMN decision
   * @param columnCount Number of the current column which is created
   * @param rowCount Number of the current row which is created
   * @return The generated input entry
   */
  public static InputEntry generateInputEntry(String identifier, int columnCount, int rowCount) {
    return new InputEntry(
        "inputEntry_" + identifier + "_" + rowCount + "_" + columnCount, generateInputRange());
  }
}
