/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.generator.entry;

import de.uniko.bks.dom.dmn.entry.OutputEntry;
import java.util.Random;

/** Class which generates a single output entry for the DMN */
public class OutputEntryGenerator {
  /**
   * Function which negates a given value
   *
   * @param outputEntry Value which should be negated
   * @return THe negated value
   */
  private static String negateOutputEntry(String outputEntry) {
    return "not(" + outputEntry + ")";
  }

  /**
   * Function which generates a random output entry using the items from the OutputEntryList
   *
   * @param identifier Identifier suffix for all ids in the DMN decision
   * @param rowCount Number of the current row which is created
   * @param outputEntryList List of output entries used in the dmn file (note the negations are
   *     created automatically)
   * @return A random output entry
   */
  public static OutputEntry generateOutputEntry(
      String identifier, int rowCount, OutputEntryList outputEntryList) {
    OutputEntry outputEntry = new OutputEntry();
    outputEntry.setId("outputEntry_" + identifier + "_" + rowCount);
    String outputEntryValue = outputEntryList.getRandomValue();
    // Check if the value should be negated or not
    if (new Random().nextBoolean()) {
      //  outputEntryValue = negateOutputEntry(outputEntryValue);
    }

    // Set the value of the output entry
    outputEntry.setText(outputEntryValue);
    return outputEntry;
  }
}
