/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.generator.header;

import de.uniko.bks.dom.dfl.rule.RuleType;
import de.uniko.bks.dom.dmn.Input;
import de.uniko.bks.dom.dmn.header.InputHeader;
import java.util.ArrayList;

/** Class which generates a list of input headers for the DMN */
public class InputHeaderGenerator {
  /**
   * Function which generates the input list (list of input headers)
   *
   * @param identifier Identifier suffix for all ids in the DMN decision
   * @param columnCount Number of columns in the DMN table
   * @return The generated input list
   */
  public static ArrayList<Input> generateInputList(String identifier, int columnCount) {
    ArrayList<Input> inputList = new ArrayList<>();
    // Generate an integer input header for each column
    for (int i = 0; i < columnCount; i++) {
      Input input = new Input();
      input.setId("input_" + identifier + "_" + i);
      input.setLabel("input_" + identifier + "_" + i);
      // Create a new input header
      InputHeader inputHeader =
          new InputHeader(
              "inputHeader_" + identifier + "_" + i,
              RuleType.INT.getValue(),
              "inputHeader_" + identifier + "_" + i);
      input.setInputHeader(inputHeader);
      inputList.add(input);
    }

    return inputList;
  }
}
