/*
 * Developed by Hans-Henning Ramberger as part of a master thesis.
 * Copyright (c) 2018.
 */

package de.uniko.bks.generator.header;

import de.uniko.bks.dom.dfl.rule.RuleType;
import de.uniko.bks.dom.dmn.HitPolicy;
import de.uniko.bks.dom.dmn.header.OutputHeader;
import de.uniko.bks.dom.dmn.header.OutputValues;
import de.uniko.bks.generator.entry.OutputEntryList;

/** Class which generates a single output header for the DMN */
public class OutputHeaderGenerator {
  /**
   * Function which generates a DMN output header
   *
   * @param identifier Identifier suffix for all ids in the DMN decision
   * @param hitPolicy The HitPolicy of the table
   * @param outputEntryList List of output entries used in the dmn file (note the negations are
   *     created automatically)
   * @return The DMN output header
   */
  public static OutputHeader generateOutputHeader(
      String identifier, String hitPolicy, OutputEntryList outputEntryList) {
    OutputHeader outputHeader = new OutputHeader();
    outputHeader.setId("output_" + identifier);
    outputHeader.setType(RuleType.STRING.getValue());
    outputHeader.setText("output_" + identifier);
    // Set the rule order if it is needed
    if (hitPolicy.equals(HitPolicy.PRIORITY.getValue())
        || hitPolicy.equals(HitPolicy.OUTPUT_ORDER.getValue())) {
      StringBuilder stringBuilder = new StringBuilder();
      for (int i = 0; i < outputEntryList.getValues().length; i++) {
        stringBuilder.append("\"");
        stringBuilder.append(outputEntryList.getValues()[i]);
        stringBuilder.append("\"");
        // Check if it is the last item
        if (i < outputEntryList.getValues().length - 1) {
          stringBuilder.append(",");
        }
      }
      outputHeader.setOutputValues(new OutputValues(identifier, stringBuilder.toString()));
    }
    return outputHeader;
  }
}
