package de.uniko.bks.statistic;

import com.google.common.collect.Multimap;
import com.google.common.collect.MultimapBuilder;
import java.util.*;

public class NumberLine {
  private ArrayList<Map<Integer, Set<dmnInt>>> line = new ArrayList<>();
  private Set<Integer> isInLine = new HashSet<>();
  private Set<dmnInt> beginning = new HashSet<>();
  private Set<dmnInt> end = new HashSet<>();
  private Set<dmnInt> all = new HashSet<>();
  private Map<dmnInt, Set<dmnInt>> numberOverlaps = new HashMap<>();
  private boolean somethingChanged = false;
  private boolean overlapsAreComputed = false;
  private Set<dmnInt> forSureOverlaps = new HashSet<>();

  private Map<dmnInt, String> mapping = new HashMap<>();

  public ArrayList<Map<Integer, Set<dmnInt>>> getLine() {
    return line;
  }

  public Set<dmnInt> computeOverlapping(dmnInt number, Set<dmnInt> setToCompute) {
    HashSet<dmnInt> result = new HashSet<>();
    for (dmnInt numberToCheck : setToCompute) {
      if (number.compare(numberToCheck)) {
        result.add(numberToCheck);
      }
    }
    return result;
  }

  public Set<Set<String>> getRuleOverlaps() {
    Set<Set<String>> result = new HashSet<>();
    if (somethingChanged || !overlapsAreComputed) {
      numberOverlaps = getOverlappingNumbers();
    }
    for (dmnInt number : numberOverlaps.keySet()) {
      String numberRule = mapping.get(number);
      for (dmnInt overlapNumber : numberOverlaps.get(number)) {
        HashSet<String> overlap = new HashSet<>();
        overlap.add(numberRule);
        overlap.add(mapping.get(overlapNumber));
        result.add(overlap);
      }
    }
    return result;
  }

  public Set<Set<String>> getRuleOverlaps(Set<Set<String>> currentIncons) {
    Set<Set<String>> result = getRuleOverlaps();
    result.retainAll(currentIncons);
    return result;
  }

  public Map<dmnInt, Set<dmnInt>> getOverlappingNumbers() {
    HashSet<dmnInt> allBefore = new HashSet<>();
    HashSet<dmnInt> allAfter = new HashSet<>(all);
    Multimap<dmnInt, dmnInt> numbersToProcess = MultimapBuilder.hashKeys().hashSetValues().build();
    Map<dmnInt, Set<dmnInt>> result = new HashMap<>();

    Iterator<Map<Integer, Set<dmnInt>>> iter = line.iterator();
    while (iter.hasNext()) {
      Map<Integer, Set<dmnInt>> currentmap = iter.next();
      if (currentmap.size() != 1) {
        System.out.println("something went wrong");
        continue;
      }
      allAfter.removeAll(currentmap.get(currentmap.keySet().iterator().next()));
      for (dmnInt currentNumber : currentmap.get(currentmap.keySet().iterator().next())) {

        if (currentNumber.isDoesntMatter()) {
          HashSet<dmnInt> allMinusItself = new HashSet<>(all);
          allMinusItself.remove(currentNumber);
          result.put(currentNumber, allMinusItself);
          continue;
        }
        HashSet<dmnInt> forCurrentNumber = new HashSet<>();
        forCurrentNumber.addAll(
            computeOverlapping(
                currentNumber, currentmap.get(currentmap.keySet().iterator().next())));
        forCurrentNumber.addAll(forSureOverlaps);
        if (currentNumber.isRange()) {
          if ((numbersToProcess).containsKey(currentNumber)) {
            forCurrentNumber.addAll(numbersToProcess.get(currentNumber));
            numbersToProcess.removeAll(currentmap);
          } else {
            computeOverlapping(currentNumber, currentmap.get(currentmap.keySet().iterator().next()))
                .forEach(number -> numbersToProcess.put(currentNumber, number));
          }
        }
        if (currentNumber.isLower()) {
          forCurrentNumber.addAll(allBefore);
          forCurrentNumber.addAll(beginning);
        }
        if (currentNumber.isGreater()) {
          forCurrentNumber.addAll(allAfter);
          forCurrentNumber.addAll(end);
        }
        if (!forCurrentNumber.isEmpty()) {
          if (forCurrentNumber.contains(currentNumber)) {
            forCurrentNumber.remove(currentNumber);
          }
          result.put(currentNumber, forCurrentNumber);
        }
      }
      allBefore.addAll(currentmap.get(currentmap.keySet().iterator().next()));
      numbersToProcess
          .keySet()
          .forEach(
              key ->
                  numbersToProcess.putAll(
                      key, currentmap.get(currentmap.keySet().iterator().next())));
    }
    numberOverlaps = result;
    somethingChanged = false;
    overlapsAreComputed = true;
    return result;
  }

  public void addToLine(dmnInt number, String ruleName) {
    somethingChanged = true;
    mapping.put(number, ruleName);
    all.add(number);
    number.setRuleName(ruleName);
    if (number.isDoesntMatter()) {
      forSureOverlaps.add(number);
      return;
    }

    if (number.isRange()) {
      addToLine(number.getLowerBoundInt(), number);
      addToLine(number.getUpperBoundInt(), number);
      return;
    }
    if (number.isLower()) {
      beginning.add(number);
    }
    if (number.isGreater()) {
      end.add(number);
    }
    addToLine(number.getNumberInt(), number);
  }

  private void addToLine(Integer number, dmnInt num) {
    if (line.isEmpty()) {
      Set<dmnInt> currentSet = new HashSet<>();
      currentSet.add(num);
      Map<Integer, Set<dmnInt>> newMap = new HashMap<>();
      newMap.put(number, currentSet);
      line.add(newMap);
    }
    ArrayList<Map<Integer, Set<dmnInt>>> newLine = new ArrayList<>(line);
    Iterator<Map<Integer, Set<dmnInt>>> iter = newLine.iterator();

    while (iter.hasNext()) {
      Map<Integer, Set<dmnInt>> currentmap = iter.next();
      if (currentmap.size() != 1) {
        System.out.println("something went wrong");
        continue;
      }
      int currentNumber = currentmap.keySet().iterator().next();
      if (number < currentNumber) {
        Set<dmnInt> currentSet = new HashSet<>();
        currentSet.add(num);
        Map<Integer, Set<dmnInt>> newMap = new HashMap<>();
        newMap.put(number, currentSet);
        int index = line.indexOf(currentmap);
        line.add(index, newMap);
      } else if (number == currentNumber) {
        Set<dmnInt> currentSet = currentmap.get(number);
        currentSet.add(num);
        Map<Integer, Set<dmnInt>> newMap = new HashMap<>();
        newMap.put(number, currentSet);
        int index = line.indexOf(currentmap);
        line.remove(currentmap);
        line.add(index, newMap);
        break;
      } else if ((number > currentNumber) && iter.hasNext()) {
        continue;
      } else if (!iter.hasNext()) {
        Set<dmnInt> currentSet = new HashSet<>();
        currentSet.add(num);
        Map<Integer, Set<dmnInt>> newMap = new HashMap<>();
        newMap.put(number, currentSet);
        line.add(newMap);
      }
    }
  }
}
