package de.unikoblenz.fgbks.base.builder;

import java.io.Serializable;
import java.lang.reflect.Field;

public abstract class AbstractBuilder<E, B extends AbstractBuilder<E, B>> implements Serializable {
  private static final String VALUE_FIELD_NAME = "value";

  protected E value;

  protected AbstractBuilder() {
    value = newInstance();
  }

  protected AbstractBuilder(E v) {
    value = v;
  }

  public static Field getAccessibleValueField() throws IllegalStateException {
    try {
      Field valueField = AbstractBuilder.class.getDeclaredField(VALUE_FIELD_NAME);
      valueField.setAccessible(true);
      return valueField;
    } catch (NoSuchFieldException e) {
      throw new IllegalStateException(
          String.format(
              "%s must have field with name '%s'",
              AbstractBuilder.class.getSimpleName(), VALUE_FIELD_NAME),
          e);
    }
  }

  protected abstract B thisBuilder();

  protected abstract E newInstance();

  protected abstract void validate();

  public E build() {
    validate();
    E result = value;
    value = newInstance();
    return result;
  }
}
