package de.unikoblenz.fgbks.dmn.api;

import de.unikoblenz.fgbks.base.utils.ExceptionInterceptor;
import de.unikoblenz.fgbks.base.utils.PerformanceInterceptor;
import de.unikoblenz.fgbks.dmn.beans.DmnBean;
import de.unikoblenz.fgbks.dmn.core.DmnService;
import de.unikoblenz.fgbks.dmn.core.models.VerifierType;
import java.util.Optional;
import javax.inject.Inject;
import javax.interceptor.Interceptors;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

@Path("/dmn")
@Interceptors({ExceptionInterceptor.class, PerformanceInterceptor.class})
public class DmnApi {

  private static final String NO_CURRENT_DMN_FOUND = "No current DMN found";

  @Inject private DmnBean dmnBean;

  @GET
  @Produces(MediaType.TEXT_PLAIN)
  @Path("/tables")
  public Response getDmnTableNames() {
    return Response.status(Status.OK).entity(createDmnService().get().getDecisionNames()).build();
  }

  @GET
  @Produces(MediaType.APPLICATION_XML)
  @Path("/getCurrentDmn")
  public Response getCurrentDmn() {
    if (dmnBean.getDmnXML() == null || dmnBean.getDmnXML().isEmpty()) {
      throw new IllegalStateException(NO_CURRENT_DMN_FOUND);
    }
    return Response.status(Status.OK).entity(dmnBean.getDmnXML()).build();
  }

  @GET
  @Produces({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON})
  @Consumes({MediaType.TEXT_PLAIN})
  @Path("/getVerification/{verifierType}")
  public Response getDmnErrors(@PathParam("verifierType") String verifierT) {
    Optional<DmnService> dmnHandler = createDmnService();
    if (dmnHandler.isPresent()) {
      VerifierType verifierType = VerifierType.valueOf(verifierT);
      return Response.status(Status.OK)
          .entity(dmnHandler.get().getVerifierFromType(verifierType))
          .build();
    }
    return Response.serverError().entity(NO_CURRENT_DMN_FOUND).build();
  }

  @GET
  @Produces({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON})
  @Path("/getVerification/all")
  public Response getAllDmnVerifications() {
    Optional<DmnService> dmnHandler = createDmnService();
    if (dmnHandler.isPresent()) {
      return Response.status(Status.OK).entity(dmnHandler.get().getAllVerifier()).build();
    }
    return Response.serverError().entity(NO_CURRENT_DMN_FOUND).build();
  }

  @POST
  @Consumes(MediaType.TEXT_XML)
  @Produces({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON})
  @Path("/getVerificationsFromDmn")
  public Response setDmnAndGetAllVerifications(String payload) {
    dmnBean.setDmnAsXmlString(payload);
    return getAllDmnVerifications();
  }

  @POST
  @Consumes(MediaType.TEXT_XML)
  @Produces(MediaType.APPLICATION_XML)
  @Path("/setCurrentDmn")
  public Response updateCurrentDmn(String payload) {
    dmnBean.setDmnAsXmlString(payload);
    return Response.status(Status.OK).entity(dmnBean.getDmnXML()).build();
  }

  private Optional<DmnService> createDmnService() {
    if (dmnBean.getDmnXML() == null || dmnBean.getDmnXML().isEmpty()) {
      return Optional.empty();
    }
    return Optional.of(DmnService.getDmnServiceFromXml(dmnBean.getDmnXML()));
  }
}
