package de.unikoblenz.fgbks.dmn.beans;

import java.io.InputStream;
import java.io.Serializable;
import javax.enterprise.context.SessionScoped;
import javax.inject.Inject;
import javax.inject.Named;
import org.apache.commons.io.IOUtils;
import org.primefaces.model.DefaultStreamedContent;
import org.primefaces.model.StreamedContent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Named
@SessionScoped
public class ButtonBean implements Serializable {

  private static Logger LOGGER = LoggerFactory.getLogger(ButtonBean.class.getSimpleName());
  private static final String SAMPLE_DMN = "sampleDMN.dmn";
  private static final String EMPTY_DMN = "emptyDMN.dmn";

  private boolean calculateMultiTable;

  @Inject private DmnBean dmnBean;

  @Inject private TableDataBean tableDataBean;

  public boolean isCalculateMultiTable() {
    return calculateMultiTable;
  }

  public void setCalculateMultiTable(boolean calculateMultiTable) {
    this.calculateMultiTable = calculateMultiTable;
  }

  public String setEmptyDmn() {
    LOGGER.info("Set empty DMN");
    InputStream inputStream = getClass().getClassLoader().getResourceAsStream(EMPTY_DMN);
    dmnBean.setDmnAsXmlString(inputStream);
    validateDmn();
    return "index";
  }

  public String validateDmn() {
    LOGGER.info("Validate DMN");
    try {
      tableDataBean.calculateVerifierResults(calculateMultiTable);
    } catch (Exception e) {
    }
    return "index";
  }

  public String setSampleDmn() {
    LOGGER.info("Set sample DMN");
    InputStream inputStream = getClass().getClassLoader().getResourceAsStream(SAMPLE_DMN);
    dmnBean.setDmnAsXmlString(inputStream);
    validateDmn();
    return "index";
  }

  public StreamedContent getDownloadDmn() {
    LOGGER.info("Download DMN");
    InputStream stream = IOUtils.toInputStream(dmnBean.getDmnXML());
    return new DefaultStreamedContent(stream, "text/xml", "decision.dmn");
  }

  public boolean isDisabled() {
    return dmnBean.getDmnXML() == null || dmnBean.getDmnXML().isEmpty();
  }
}
