package de.unikoblenz.fgbks.dmn.core;

import de.unikoblenz.fgbks.dmn.core.models.VerifierCollectionResult;
import de.unikoblenz.fgbks.dmn.core.models.VerifierResult;
import de.unikoblenz.fgbks.dmn.core.models.VerifierType;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import org.apache.commons.io.IOUtils;
import org.camunda.bpm.dmn.engine.DmnDecision;
import org.camunda.bpm.dmn.engine.DmnEngineConfiguration;

public class DmnService {

  private final List<DmnDecision> dmnDecisions;

  public static DmnService getDmnServiceFromXml(String dmnXmlString) {
    return getDmnServiceFromXmlStream(IOUtils.toInputStream(dmnXmlString));
  }

  public static DmnService getDmnServiceFromXmlStream(InputStream dmnInputStream) {
    return new DmnService(
        DmnEngineConfiguration.createDefaultDmnEngineConfiguration()
            .buildEngine()
            .parseDecisions(dmnInputStream));
  }

  public DmnService(List<DmnDecision> dmnDecisions) {
    this.dmnDecisions = new ArrayList<>(Objects.requireNonNull(dmnDecisions));
  }

  public List<DmnDecision> getDmnDecisions() {
    return new ArrayList<>(dmnDecisions);
  }

  public String getDecisionNames() {
    StringBuilder sb = new StringBuilder();
    for (DmnDecision d : dmnDecisions) {
      sb.append(d.getName()).append(System.lineSeparator());
    }
    return sb.toString();
  }

  public VerifierResult getVerifierFromType(VerifierType verifierType) {
    return verifierType.getResult(dmnDecisions).orElse(null);
  }

  public VerifierCollectionResult getAllVerifier() {
    return VerifierType.getAllResults(dmnDecisions);
  }

  public VerifierCollectionResult getAllVerifier(boolean calculateMultiTable) {
    return VerifierType.getAllResults(dmnDecisions, calculateMultiTable);
  }
}
