package de.unikoblenz.fgbks.dmn.core.dmnhelper;

import org.camunda.bpm.dmn.engine.impl.DmnExpressionImpl;

public enum DataType {
  FLOATING(true),
  INTEGER(true),
  BOOLEAN(false),
  STRING(false),
  DATE(false),
  UNKNOWN(false);

  private boolean numeric = false;

  private DataType(boolean numeric) {
    this.numeric = numeric;
  }

  public boolean isNumeric() {
    return numeric;
  }

  public static DataType getTypeFromString(String type) {
    switch (type) {
      case "integer":
      case "long":
        return DataType.INTEGER;
      case "double":
      case "float":
        return DataType.FLOATING;
      case "string":
        return DataType.STRING;
        // case "date":
        //  this.dataType = DataType.DATE;
        //  break;
      case "boolean":
        return DataType.BOOLEAN;
      default:
        return DataType.UNKNOWN;
    }
  }

  public Boundary getBoundaryValue(DmnExpressionImpl value) {
    if (value.getExpression() == null) {
      return Boundary.parseBoundaryFromNullValue();
    }
    switch (this) {
      case INTEGER:
        return Boundary.parseFromNumericValue(value.getExpression(), true);
      case FLOATING:
        return Boundary.parseFromNumericValue(value.getExpression());
      case BOOLEAN:
        return Boundary.parseFromBooleanValue(Boolean.valueOf(value.getExpression()));
      case STRING:
        return Boundary.parseFromStringValue(value.getExpression());
      case DATE: // not implemented yet
      case UNKNOWN: // not implemented yet
      default:
        throw new IllegalStateException(
            String.format("Parsing boundary of value %s failed.", value.getExpression()));
    }
  }
}
