package de.unikoblenz.fgbks.dmn.core.dmnhelper;

import de.unikoblenz.fgbks.dmn.core.models.RuleIdentifier;
import java.util.Objects;
import org.camunda.bpm.dmn.engine.DmnDecision;
import org.camunda.bpm.dmn.engine.impl.DmnExpressionImpl;

public abstract class Type {

  protected final DmnDecision decision;
  protected final String id;
  protected final DataType dataType;
  protected final String label;
  protected String overwrittenDecisionKey;

  public String getDecisionKey() {
    if (overwrittenDecisionKey != null) {
      return overwrittenDecisionKey;
    }
    return decision.getKey();
  }

  public DmnDecision getDecision() {
    return decision;
  }

  public String getId() {
    return id;
  }

  public DataType getDataType() {
    return dataType;
  }

  public String getLabel() {
    return label;
  }

  public void setOverwrittenDecisionKey(String overwrittenDecisionKey) {
    this.overwrittenDecisionKey = overwrittenDecisionKey;
  }

  protected Type(DmnDecision decision, String id, DataType dataType, String label) {
    this.decision = decision;
    this.id = id;
    this.dataType = dataType;
    this.label = label;
    this.overwrittenDecisionKey = null;
  }

  public Type(Type type) {
    this.decision = type.decision;
    this.id = type.id;
    this.dataType = type.dataType;
    this.label = type.label;
    this.overwrittenDecisionKey = type.overwrittenDecisionKey;
  }

  public boolean isMergedType() {
    return overwrittenDecisionKey != null;
  }

  public Value getValue(
      DmnExpressionImpl value, DmnDecision decision, int rowNumber, String ruleId) {
    return new Value(
        dataType.getBoundaryValue(value),
        value.getExpression(),
        RuleIdentifier.getBuilder()
            .withDecisionName(decision.getName())
            .withDecisionKey(decision.getKey())
            .withRowNumber(rowNumber)
            .withRuleId(ruleId)
            .withConditionId(value.getId())
            .withConditionName(value.getName())
            .build(),
        this);
  }

  public boolean isEqualExpression(Type type) {
    return this.label.equals(type.label);
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    Type type = (Type) o;
    return getDecisionKey().equals(type.getDecisionKey())
        && id.equals(type.id)
        && dataType == type.dataType;
  }

  @Override
  public int hashCode() {
    return Objects.hash(getDecisionKey(), id, dataType);
  }
}
