package de.unikoblenz.fgbks.dmn.core.models;

import de.unikoblenz.fgbks.base.builder.DefaultBuilder;
import java.io.Serializable;
import java.util.Objects;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import org.camunda.bpm.dmn.engine.DmnDecision;

@XmlRootElement(name = "rule")
public class RuleIdentifier implements Serializable {

  private String decisionKey;
  private String decisionName;
  private String ruleId;
  private Integer rowNumber;
  private String conditionId;
  private String conditionName;

  protected RuleIdentifier() {
    super();
  }

  @XmlElement
  public String getDecisionKey() {
    return decisionKey;
  }

  private void setDecisionKey(String decisionKey) {
    this.decisionKey = decisionKey;
  }

  @XmlElement
  public String getDecisionName() {
    return decisionName;
  }

  private void setDecisionName(String decisionName) {
    this.decisionName = decisionName;
  }

  public String getTableName() {
    return decisionName != null ? decisionName : decisionKey;
  }

  @XmlElement
  public String getRuleId() {
    return ruleId;
  }

  private void setRuleId(String ruleId) {
    this.ruleId = ruleId;
  }

  @XmlElement
  public Integer getRowNumber() {
    return rowNumber;
  }

  private void setRowNumber(Integer rowNumber) {
    this.rowNumber = rowNumber;
  }

  @XmlElement
  public String getConditionId() {
    return conditionId;
  }

  private void setConditionId(String conditionId) {
    this.conditionId = conditionId;
  }

  @XmlElement
  public String getConditionName() {
    return conditionName;
  }

  public void setConditionName(String conditionName) {
    this.conditionName = conditionName;
  }

  public static Builder getBuilder() {
    return new RuleIdentifier().new Builder();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    RuleIdentifier that = (RuleIdentifier) o;
    return decisionKey.equals(that.decisionKey) && ruleId.equals(that.ruleId);
  }

  @Override
  public int hashCode() {
    return Objects.hash(decisionKey, ruleId);
  }

  public class Builder extends DefaultBuilder<RuleIdentifier> {

    private Builder() {}

    public Builder withDecision(DmnDecision d) {
      value.setDecisionKey(d.getKey());
      value.setDecisionName(d.getName());
      return this;
    }

    public Builder withDecisionKey(String decisionKey) {
      value.setDecisionKey(decisionKey);
      return this;
    }

    public Builder withDecisionName(String decisionName) {
      value.setDecisionName(decisionName);
      return this;
    }

    public Builder withRuleId(String ruleId) {
      value.setRuleId(ruleId);
      return this;
    }

    public Builder withRowNumber(Integer rowNumber) {
      value.setRowNumber(rowNumber);
      return this;
    }

    public Builder withConditionId(String conditionId) {
      value.setConditionId(conditionId);
      return this;
    }

    public Builder withConditionName(String conditionName) {
      value.setConditionName(conditionName);
      return this;
    }

    @Override
    protected void validate() {
      Objects.requireNonNull(value.decisionKey);
      Objects.requireNonNull(value.decisionName);
      Objects.requireNonNull(value.ruleId);
      Objects.requireNonNull(value.rowNumber);
    }
  }
}
