package de.unikoblenz.fgbks.dmn.core.models;

import de.unikoblenz.fgbks.base.builder.DefaultBuilder;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import javax.json.Json;
import javax.json.JsonArrayBuilder;
import javax.json.JsonObjectBuilder;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

@XmlRootElement(name = "verification")
public class VerificationResult implements Serializable {

  @XmlElement(name = "rule")
  private List<RuleIdentifier> rules;

  @XmlElement(name = "message")
  private String message;

  protected VerificationResult() {
    super();
    rules = new ArrayList<>();
  }

  @XmlElement(name = "ruleCount")
  public int getRuleCount() {
    return rules.size();
  }

  public List<RuleIdentifier> getRules() {
    return new ArrayList<>(rules);
  }

  private void setMessage(String message) {
    this.message = message;
  }

  public String getMessage() {
    return message;
  }

  public String getRulesAsJson() {
    JsonObjectBuilder result = Json.createObjectBuilder();
    JsonArrayBuilder jsonRules = Json.createArrayBuilder();
    rules.stream()
        .map(
            r ->
                Json.createObjectBuilder()
                    .add("ruleId", r.getRuleId())
                    .add("decisionKey", r.getDecisionKey())
                    .add("rowNumber", r.getRowNumber())
                    .build())
        .forEach(jsonRules::add);
    result.add("rules", jsonRules);
    result.add("ruleCount", getRuleCount());
    String s = result.build().toString();
    return s;
  }

  public static VerificationResult.Builder getBuilder() {
    return new VerificationResult().new Builder();
  }

  public class Builder extends DefaultBuilder<VerificationResult> {

    public Builder addRule(RuleIdentifier ruleIdentifier) {
      value.rules.add(ruleIdentifier);
      return this;
    }

    public Builder addRules(Collection<RuleIdentifier> ruleIdentifier) {
      value.rules.addAll(ruleIdentifier);
      return this;
    }

    public Builder withMessage(String message, Object... args) {
      value.setMessage(String.format(message, args));
      return this;
    }

    public Builder withMessage(String message) {
      value.setMessage(message);
      return this;
    }

    @Override
    protected void validate() {
      Objects.requireNonNull(value.message);
    }
  }
}
