package de.unikoblenz.fgbks.dmn.core.models;

import de.unikoblenz.fgbks.base.builder.DefaultBuilder;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

@XmlRootElement(name = "verifierResult")
public class VerifierResult implements Serializable {

  @XmlElement private VerifierType verifierType;

  @XmlElement
  public String getVerificationDescription() {
    return verifierType.getDescription();
  }

  @XmlElementWrapper(name = "verifications")
  @XmlElement(name = "verification")
  private Set<VerificationResult> verifications;

  protected VerifierResult() {
    super();
    verifications = new HashSet<>();
  }

  @XmlElement(name = "verificationsCount")
  public int getVerificationsCount() {
    return verifications.size();
  }

  public VerifierType getVerifierType() {
    return verifierType;
  }

  public Set<VerificationResult> getVerifications() {
    return new HashSet<>(verifications);
  }

  private void setVerifierType(VerifierType verifierType) {
    this.verifierType = verifierType;
  }

  public static VerifierResult.Builder getBuilder() {
    return new VerifierResult().new Builder();
  }

  public class Builder extends DefaultBuilder<VerifierResult> {

    public Builder withVerifierType(VerifierType verifierType) {
      value.setVerifierType(verifierType);
      return this;
    }

    public Builder addVerificationResult(VerificationResult ruleIdentifier) {
      value.verifications.add(ruleIdentifier);
      return this;
    }

    @Override
    protected void validate() {
      Objects.requireNonNull(value.verifierType);
    }
  }
}
