package de.unikoblenz.fgbks.dmn.core.verifier;

import de.unikoblenz.fgbks.base.wordnet.WordnetService;
import de.unikoblenz.fgbks.dmn.core.dmnhelper.DataType;
import de.unikoblenz.fgbks.dmn.core.dmnhelper.Type;
import de.unikoblenz.fgbks.dmn.core.dmnhelper.Value;
import de.unikoblenz.fgbks.dmn.core.models.RuleIdentifier;
import de.unikoblenz.fgbks.dmn.core.models.VerificationResult;
import java.util.List;
import java.util.stream.Collectors;
import org.camunda.bpm.dmn.engine.DmnDecision;

public class EquivalentVerifier extends AbstractVerifier {

  private WordnetService wns = WordnetService.getInstance();

  @Override
  protected void beforeVerifyDecision() {
    // Nothing
  }

  @Override
  protected void verifyDecision(DmnDecision d) {
    if (d.isDecisionTable()) {
      findEquivalentRules(dmnRuleMap.getAllTypesFromDecisionKey(d.getKey()));
    }
  }

  protected void findEquivalentRules(List<Type> allTypesFromDecisionKey) {
    for (Type t : allTypesFromDecisionKey) {
      // only check strings for equvalent rules (input + output)
      if (t.getDataType() == DataType.STRING) {
        // ensure label is not null and not empty
        List<Value> values =
            dmnRuleMap.getValuesFromType(t).stream()
                .filter(
                    v ->
                        v.getOrgExpression() != null
                            && !v.getOrgExpression().isEmpty()
                            && !v.getOrgExpression().equals("\"\""))
                .collect(Collectors.toList());
        // Check all combinations for synonyms, if found -> add both rules to the message
        for (int i = 0; i < values.size() - 1; i++) {
          for (int u = i + 1; u < values.size(); u++) {
            String val1 = values.get(i).getOrgExpression().replace("\"", "");
            String val2 = values.get(u).getOrgExpression().replace("\"", "");
            if (!val1.equals(val2) && wns.areNounsSynonyms(val1, val2)) {
              addVerification(
                  VerificationResult.getBuilder()
                      .addRule(values.get(i).getRuleIdentifier())
                      .addRule(values.get(u).getRuleIdentifier())
                      .withMessage(
                          getMessageText(
                              values.get(i).getRuleIdentifier(),
                              values.get(u).getRuleIdentifier(),
                              val1,
                              val2))
                      .build());
            }
          }
        }
      }
    }
  }

  protected String getMessageText(RuleIdentifier r1, RuleIdentifier r2, String v1, String v2) {
    return String.format(
        "In rule %d string \"%s\" and in rule %d string \"%s\": equal meaning? Are they synonyms?",
        r1.getRowNumber(), v1, r2.getRowNumber(), v2);
  }

  @Override
  protected void afterVerifyDecision() {
    // Nothing
  }
}
