package de.unikoblenz.fgbks.dmn.core.verifier;

import de.unikoblenz.fgbks.dmn.core.dmnhelper.Type;
import de.unikoblenz.fgbks.dmn.core.dmnhelper.Value;
import de.unikoblenz.fgbks.dmn.core.models.RuleIdentifier;
import de.unikoblenz.fgbks.dmn.core.models.VerificationResult;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import org.camunda.bpm.dmn.engine.DmnDecision;

public class IdenticalVerifier extends AbstractVerifier {

  @Override
  protected void beforeVerifyDecision() {
    // Nothing
  }

  @Override
  protected void verifyDecision(DmnDecision d) {
    if (d.isDecisionTable()) {
      List<Type> inputs = new ArrayList<>(dmnRuleMap.getAllInputTypesFromDecisionKey(d.getKey()));
      checkForIdenticalRules(inputs, 0, null);
    }
  }

  protected void checkForIdenticalRules(
      List<Type> inputs, int i, List<RuleIdentifier> currentRuleIdentifiers) {
    if (i == inputs.size()) {
      boolean differentConclusions =
          checkDifferentConclusion(inputs.get(0).getDecisionKey(), currentRuleIdentifiers);
      VerificationResult.Builder vBuilder =
          VerificationResult.getBuilder()
              .withMessage(getMessageText(currentRuleIdentifiers, differentConclusions));
      vBuilder.addRules(currentRuleIdentifiers);
      VerificationResult vResult = vBuilder.build();
      if (differentConclusions) {
        interdeterminismRules.add(vResult);
      }
      addVerification(vResult);
    } else {
      List<Value> currentBounds = new ArrayList<>();
      List<Value> sortedBounds =
          dmnRuleMap.getValuesFromInputType(inputs.get(i), currentRuleIdentifiers);
      sortedBounds.sort(Comparator.comparing(Value::getBoundary));
      Value lastBound = null;
      for (Value currentBound : sortedBounds) {
        if (lastBound != null) {
          if (currentBound.getBoundary().isEqualsTo(lastBound.getBoundary())) {
            currentBounds.add(lastBound);
          } else {
            if (currentBounds.size() > 0) {
              currentBounds.add(lastBound);
              checkForIdenticalRules(
                  inputs,
                  i + 1,
                  currentBounds.stream()
                      .map(Value::getRuleIdentifier)
                      .collect(Collectors.toList()));
            }
            currentBounds.clear();
          }
        }
        lastBound = currentBound;
      }
      if (currentBounds.size() > 0) {
        currentBounds.add(lastBound);
        checkForIdenticalRules(
            inputs,
            i + 1,
            currentBounds.stream().map(Value::getRuleIdentifier).collect(Collectors.toList()));
      }
    }
  }

  protected String getMessageText(
      List<RuleIdentifier> currentRuleIdentifiers, boolean isDifferentConclusion) {
    StringBuilder sb = new StringBuilder("Rules ");
    sb.append(
        currentRuleIdentifiers.stream()
            .map(c -> c.getRowNumber().toString())
            .collect(Collectors.joining(", ")));
    sb.append(" in table ");
    sb.append(currentRuleIdentifiers.get(0).getTableName());
    sb.append(" are identical.");
    if (isDifferentConclusion) {
      sb.append(" The output has different conclusions!");
    } else {
      sb.append(" The output is the same.");
    }
    return sb.toString();
  }

  @Override
  protected void afterVerifyDecision() {
    // Nothing}
  }
}
