package de.unikoblenz.fgbks.dmn.core.verifier.multitable;

import de.unikoblenz.fgbks.dmn.core.dmnhelper.Type;
import de.unikoblenz.fgbks.dmn.core.models.RuleIdentifier;
import de.unikoblenz.fgbks.dmn.core.verifier.IdenticalVerifier;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import org.camunda.bpm.dmn.engine.DmnDecision;

public class MultiTableIdenticalVerifier extends IdenticalVerifier {

  @Override
  protected void beforeVerifyDecision() {
    for (String key : dmnRuleMap.getMergedDecisionsKeys()) {
      List<Type> inputs = new ArrayList<>(dmnRuleMap.getAllInputTypesFromDecisionKey(key));
      checkForIdenticalRules(inputs, 0, null);
    }
  }

  @Override
  protected void verifyDecision(DmnDecision d) {
    // nothing
  }

  @Override
  protected void afterVerifyDecision() {
    super.afterVerifyDecision();
  }

  @Override
  protected String getMessageText(
      List<RuleIdentifier> currentRuleIdentifiers, boolean isDifferentConclusion) {
    StringBuilder sb = new StringBuilder("Rules ");
    sb.append(
        currentRuleIdentifiers.stream()
            .map(c -> c.getRowNumber().toString() + " (" + c.getTableName() + ")")
            .collect(Collectors.joining(", ")));
    sb.append(" are identical.");
    if (isDifferentConclusion) {
      sb.append(" The output has different conclusions!");
    } else {
      sb.append(" The output is the same.");
    }
    return sb.toString();
  }
}
