package de.unikoblenz.fgbks.dmn.core.verifier.multitable;

import de.unikoblenz.fgbks.dmn.core.dmnhelper.Type;
import de.unikoblenz.fgbks.dmn.core.models.RuleIdentifier;
import de.unikoblenz.fgbks.dmn.core.verifier.PartialReductionVerifier;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import org.camunda.bpm.dmn.engine.DmnDecision;

public class MultiTablePartialReductionVerifier extends PartialReductionVerifier {

  @Override
  protected void beforeVerifyDecision() {
    for (String key : dmnRuleMap.getMergedDecisionsKeys()) {
      List<List<RuleIdentifier>> identicalOutputCluster = findIdenticalOutputs(key);
      List<Type> inputs = new ArrayList<>(dmnRuleMap.getAllInputTypesFromDecisionKey(key));
      identicalOutputCluster.stream() // do: parallelStream() ?
          .forEach(c -> findPartialReduction(inputs, 0, c, false));
    }
  }

  @Override
  protected void verifyDecision(DmnDecision d) {
    // nothing
  }

  @Override
  protected void afterVerifyDecision() {
    // Nothing
  }

  @Override
  protected String getMessageText(List<RuleIdentifier> ruleIdentifiers) {
    StringBuilder sb = new StringBuilder("Rules ");
    sb.append(
        ruleIdentifiers.stream()
            .map(c -> c.getRowNumber().toString() + " (" + c.getTableName() + ")")
            .collect(Collectors.joining(" and ")));
    sb.append(" can be combined.");
    return sb.toString();
  }
}
