package de.unikoblenz.fgbks.dmn.core.verifier.multitable;

import de.unikoblenz.fgbks.dmn.core.dmnhelper.Type;
import de.unikoblenz.fgbks.dmn.core.dmnhelper.Value;
import de.unikoblenz.fgbks.dmn.core.models.RuleIdentifier;
import de.unikoblenz.fgbks.dmn.core.verifier.SubsumptionVerifier;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import org.camunda.bpm.dmn.engine.DmnDecision;

public class MultiTableSubsumptionVerifier extends SubsumptionVerifier {

  @Override
  protected void beforeVerifyDecision() {
    for (String key : dmnRuleMap.getMergedDecisionsKeys()) {
      List<Type> inputs = new ArrayList<>(dmnRuleMap.getAllInputTypesFromDecisionKey(key));
      checkForSubsumptionRules(inputs, 0, null, false, null);
    }
  }

  @Override
  protected void verifyDecision(DmnDecision d) {
    // nothing
  }

  @Override
  protected void afterVerifyDecision() {
    // Nothing
  }

  protected String getMessageText(
      List<RuleIdentifier> currentRuleIdentifiers,
      List<Value> subsumptionRules,
      boolean isDifferentConclusion) {
    StringBuilder sb = new StringBuilder();
    sb.append("In table ");
    sb.append(subsumptionRules.get(0).getRuleIdentifier().getTableName());
    sb.append(" rule");
    if (subsumptionRules.size() > 1) {
      sb.append("s");
    }
    sb.append(" ");
    sb.append(
        subsumptionRules.stream()
            .map(c -> c.getRuleIdentifier().getRowNumber().toString())
            .sorted()
            .collect(Collectors.joining(", ")));
    sb.append(" subsumes rule");
    if (currentRuleIdentifiers.size() > 2) {
      sb.append("s");
    }
    sb.append(" ");
    sb.append(
        currentRuleIdentifiers.stream()
            .filter(
                r ->
                    !subsumptionRules.stream()
                        .map(Value::getRuleIdentifier)
                        .collect(Collectors.toList())
                        .contains(r))
            .map(c -> c.getRowNumber().toString() + " (" + c.getTableName() + ")")
            .sorted()
            .collect(Collectors.joining(", ")));
    sb.append(".");
    if (isDifferentConclusion) {
      sb.append(" The output has different conclusions!");
    } else {
      sb.append(" The output is the same.");
    }
    return sb.toString();
  }
}
